#/usr/bin/perl -w
<<DOC;
Marie Garrigue
Sandy Bonin
Juin 2010

utilisation de leur script avec modification des noms des diffrents fichiers de sortie

commande : perl bao3_treetagger.pl fichier_tag_xml fichier_motif

DOC
#------------------------------------------------------------------
#On utilise le module basename
use File::Basename;
#-----------------------------------------------------------------------------------------------
#On utilise le module XML::XPath
use XML::XPath;
#-----------------------------------------------------------------------------------------------
#On vrifie le nombre d'arguments de l'appel au script ("$0" est la variable qui contient le nom du script)
if($#ARGV!=1){print "commande : perl $0 fichier_tag_xml fichier_motif";exit;}
#-----------------------------------------------------------------------------------------------
#On enregistre les arguments dans les variables idoines
my $tag_file=shift @ARGV;
my $patterns_file=shift @ARGV;
#-----------------------------------
#On retire l'extension du fichier appel
my $nom=basename($tag_file,'-treetagger.xml');
print "$nom\n";

#------------------------------------------------------------
#On initialise des variables
my @patterns;
my $nb_patterns=0;
my $nb_tokens=0;
#-----------------------------------------------------------------------------------------------
#On modifie le contenu du fichier d'entre TreeTagger XML,  l'aide d'un fichier temporaire, afin qu'il puisse tre utilis par XML::XPath
#-----------------------------------------------------------------------------------------------
#On ouvre en criture le fichier temporaire, sinon on affiche un message d'erreur
open(TMP,">tmp.xml") or die "Probleme lors de l'ouverture du fichier temporaire : $!\n";
#On ouvre en lecture le fichier d'entre TreeTagger XML, sinon on affiche un message d'erreur
open(FILE,$tag_file) or die "Probleme lors de l'ouverture du fichier d'entre $tag_file : $!\n";
#On lit le fichier d'entre ligne par ligne
while ($lg=<FILE>) {
#On remplace les retours  la ligne par rien
$lg=~s/\n//g;
#On remplace la chane de caractres "NAM" par la chane de caractres "NOM"
$lg=~s/NAM/NOM/g;
#On enlve les espaces qui se trouvent avant les chevrons ouvrants
$lg=~s/ \</\</g;
#On enlve les espaces qui se trouvent aprs les chevrons fermants
$lg=~s/\> /\>/g;
#On crit la ligne  la suite du contenu du fichier temporaire
print TMP $lg;
}
#On ferme le fichier d'entre
close(FILE);
#On ferme le fichier temporaire
close(TMP);
#-----------------------------------------------------------------------------------------------
#On stocke les patrons syntaxiques dans une liste
#-----------------------------------------------------------------------------------------------
#On ouvre en lecture le fichier qui contient les patrons syntaxiques, sinon on affiche un message d'erreur
open(PATTERNSFILE,$patterns_file) or die "Probleme lors de l'ouverture du fichier d'entre $patterns_file qui contient les patrons syntaxiques : $!\n";
#On lit le fichier contenant les patrons syntaxiques, un motif par ligne (par exemple : NOM ADJ)
while ($ligne=<PATTERNSFILE>) {
#On supprime avec la commande "chomp" un ventuel retour  la ligne
chomp($ligne);
#On affecte  une variable "$nb_patterns" le nombre de patrons syntaxiques
$nb_patterns=push(@patterns,$ligne);
}
#-----------------------------------------------------------------------------------------------
#On cre un objet XML::XPath pour explorer le fichier d'entre TreeTagger XML, sinon on affiche un message d'erreur
my $xp = XML::XPath->new( filename => "tmp.xml" ) or die "Problme lors de la cration de l'objet XML::XPATH !\n";
#On recherche des patrons syntaxiques dans le fichier d'entre TreeTagger XML
foreach my $pattern (@patterns){
#On construit au moyen de la commande "split" un tableau dont chaque lment a pour valeur un token du motif recherch
@tokens=split(/ /,$pattern);
#On dfinit le nom du fichier de sortie pour le motif en utilisant la commande "join"
my $match_file="Patrons/sortieXml/X-path/liste_X-path_$nom-".join('_', @tokens).".txt";
#On ouvre en criture le fichier de sortie, sinon on affiche un message d'erreur
open(MATCHFILE,">$match_file") or die "Probleme lors de l'ouverture du fichier de sortie $match_file : $!\n";
#On appel la procdure d'extraction des formes correspondants au motif
&extract_pattern(@tokens);
#On ferme le fichier de sortie
close(MATCHFILE);
}
#-----------------------------------------------------------------------------------------------
#On cre la fonction qui permet l'extraction d'un patron syntaxique
#-----------------------------------------------------------------------------------------------
sub extract_pattern {
@tokenz=@_;
#La commande "shift" coupe le premier lement d'un tableau et le revoie en rsultat
$first_token=shift @tokenz;
#On supprime avec la commande "chomp" un ventuel retour  la ligne
chomp($first_token);
#On initialisation le chemin XPATH correspondant au motif recherch
#ATTENTION ici aux effets de bord dus  la structure choisie pour le fichier TreeTagger XML
#Par exemple pour le motif "NOM ADJ" : Une description d'un fil RSS qui se termine par un NOM suivie d'une autre description qui commence par un ADJ !
$search_path="//element/data[1][contains(text(),\"$first_token\")]";
foreach my $token (@tokenz){ #Construction rcursive du chemin XPATH correspondant au motif recherch
chomp($token);
$search_path.="/ancestor::element/following-sibling::element[1]/data[1][contains(text(),\"$token\")]";
}
#On boucle sur les noeuds reconnus du chemin XPATH
foreach my $noeud ( $xp->findnodes($search_path)->get_nodelist ) {
#On initialise un tableau des formes
#On le fait ici pour des raisons videntes d'conomie de mmoire et donc de performance
my @matching_tokens;
#On remonte d'un cran au noeud parent pour extraire la forme trouve
#Dans le cas d'un motif "NOM ADJ", c'est la forme de l'adjectif qu'on atteint
$noeud_tmp=$noeud->getParentNode;
$i=0;
foreach (@tokens){
$i++;
#On rcupre la forme
#Noter que le "3" de "getChildNode(3)" correspond au "data[3]" de la feuille XSLT
$motif=$noeud_tmp->getChildNode(3)->string_value;
#"unshift(@matching_tokens,$motif)" ajoute au dbut du tableau "@matching_tokens" un lment dont la valeur est le contenu de la variable "$motif"
$nb_tokens=unshift(@matching_tokens,$motif);
$motif="";
#On remonte rcursivement aux noeuds prcedents pour extraire la forme
#Dans le cas d'un motif "NOM PRP NOM", on rcupre ainsi PRP puis NOM (le premier)
#Noter que "preceding-sibling" est l'axe inverse de "following-sibling"
@noeudtmp=$xp->findnodes("./preceding-sibling::element[1]",$noeud_tmp)->get_nodelist;
$noeud_tmp=shift(@noeudtmp);
}
#On crit les rsultats dans un fichier en utilisant la commande "join"
print MATCHFILE join(' ', @matching_tokens)."\n";
}
}
#-----------------------------------------------------------------------------------------------
#On supprime le fichier temporaire
unlink("tmp.xml") or die "Problme lors de la suppression du fichier temporaire";
